@@ -1,4 +1,4 @@
-use std::cmp::Ordering;
+use std::{cmp::Ordering, ops::Range};
 
 use gcd::Gcd;
 use itertools::Itertools;
@@ -23,9 +23,26 @@ pub struct OptRange {
 
 impl OptRange {
     /// Returns `true` if at least one of `min` or `max` is set
-    pub fn is_set(&self) -> bool {
+    pub fn is_set(self) -> bool {
         self.min.is_some() || self.max.is_some()
     }
+
+    /// Applies [`Option::or`] to both `min` and `max`
+    pub fn or(self, other: Self) -> Self {
+        Self {
+            min: self.min.or(other.min),
+            max: self.max.or(other.max),
+        }
+    }
+
+    pub fn or_range(self, other: &Range<usize>) -> Range<usize> {
+        let min = self.min.unwrap_or(other.start);
+        let max = self
+            .max
+            .map(|x| x + 1) // +1 because `OptRange` is inclusive
+            .unwrap_or(other.end);
+        min..max
+    }
 }
 
 /// Given a set of method titles and possible shorthands, compute shorthands for the methods which
